@@ -3,7 +3,7 @@
 
 std::vector<uu::net::Network*>
 resolve_layers(
-    const uu::net::AttributedHomogeneousMultilayerNetwork* mnet,
+    const uu::net::MultilayerNetwork* mnet,
     const Rcpp::CharacterVector& names
 )
 {
@@ -41,7 +41,7 @@ resolve_layers(
 
 std::unordered_set<uu::net::Network*>
 resolve_layers_unordered(
-    const uu::net::AttributedHomogeneousMultilayerNetwork* mnet,
+    const uu::net::MultilayerNetwork* mnet,
     const Rcpp::CharacterVector& names
 )
 {
@@ -77,7 +77,7 @@ resolve_layers_unordered(
 
 std::unordered_set<const uu::net::Network*>
 resolve_const_layers_unordered(
-    const uu::net::AttributedHomogeneousMultilayerNetwork* mnet,
+    const uu::net::MultilayerNetwork* mnet,
     const Rcpp::CharacterVector& names
 )
 {
@@ -112,18 +112,18 @@ resolve_const_layers_unordered(
 
 std::vector<const uu::net::Vertex*>
 resolve_actors(
-    const uu::net::AttributedHomogeneousMultilayerNetwork* mnet,
+    const uu::net::MultilayerNetwork* mnet,
     const Rcpp::CharacterVector& names
 )
 {
-    int result_size = names.size()?names.size():mnet->vertices()->size();
+    int result_size = names.size()?names.size():mnet->actors()->size();
     std::vector<const uu::net::Vertex*> res(result_size);
 
     if (names.size()==0)
     {
         int i = 0;
 
-        for (auto actor: *mnet->vertices())
+        for (auto actor: *mnet->actors())
         {
             res[i] = actor;
             i++;
@@ -134,7 +134,7 @@ resolve_actors(
     {
         for (int i=0; i<names.size(); ++i)
         {
-            auto actor = mnet->vertices()->get(std::string(names[i]));
+            auto actor = mnet->actors()->get(std::string(names[i]));
 
             if (!actor)
             {
@@ -150,7 +150,7 @@ resolve_actors(
 
 std::unordered_set<const uu::net::Vertex*>
 resolve_actors_unordered(
-    const uu::net::AttributedHomogeneousMultilayerNetwork* mnet,
+    const uu::net::MultilayerNetwork* mnet,
     const Rcpp::CharacterVector& names
 )
 {
@@ -158,7 +158,7 @@ resolve_actors_unordered(
 
     if (names.size()==0)
     {
-        for (auto actor: *mnet->vertices())
+        for (auto actor: *mnet->actors())
         {
             res.insert(actor);
         }
@@ -168,7 +168,7 @@ resolve_actors_unordered(
     {
         for (int i=0; i<names.size(); ++i)
         {
-            auto actor = mnet->vertices()->get(std::string(names[i]));
+            auto actor = mnet->actors()->get(std::string(names[i]));
 
             if (!actor)
             {
@@ -184,7 +184,7 @@ resolve_actors_unordered(
 
 std::vector<std::pair<const uu::net::Vertex*, uu::net::Network*>>
         resolve_vertices(
-            const uu::net::AttributedHomogeneousMultilayerNetwork* mnet,
+            const uu::net::MultilayerNetwork* mnet,
             const Rcpp::DataFrame& vertex_matrix
         )
 {
@@ -194,7 +194,7 @@ std::vector<std::pair<const uu::net::Vertex*, uu::net::Network*>>
 
     for (int i=0; i<vertex_matrix.nrow(); i++)
     {
-        auto actor = mnet->vertices()->get(std::string(a(i)));
+        auto actor = mnet->actors()->get(std::string(a(i)));
 
         if (!actor)
         {
@@ -223,7 +223,7 @@ std::vector<std::pair<const uu::net::Vertex*, uu::net::Network*>>
 
 std::vector<std::tuple<const uu::net::Vertex*, uu::net::Network*, const uu::net::Vertex*, uu::net::Network*>>
         resolve_edges(
-            const uu::net::AttributedHomogeneousMultilayerNetwork* mnet,
+            const uu::net::MultilayerNetwork* mnet,
             const Rcpp::DataFrame& edges
         )
 {
@@ -235,14 +235,14 @@ std::vector<std::tuple<const uu::net::Vertex*, uu::net::Network*, const uu::net:
 
     for (int i=0; i<edges.nrow(); i++)
     {
-        auto actor1 = mnet->vertices()->get(std::string(a_from(i)));
+        auto actor1 = mnet->actors()->get(std::string(a_from(i)));
 
         if (!actor1)
         {
             Rcpp::stop("cannot find actor " + std::string(a_from(i)));
         }
 
-        auto actor2 = mnet->vertices()->get(std::string(a_to(i)));
+        auto actor2 = mnet->actors()->get(std::string(a_to(i)));
 
         if (!actor2)
         {
@@ -355,7 +355,7 @@ to_dataframe(
 std::unique_ptr<uu::net::CommunityStructure<uu::net::VertexLayerCommunity<const uu::net::Network>>>
 to_communities(
                const DataFrame& com,
-               const uu::net::AttributedHomogeneousMultilayerNetwork* mnet
+               const uu::net::MultilayerNetwork* mnet
                )
 {
     CharacterVector cs_actor = com["actor"];
@@ -368,7 +368,7 @@ to_communities(
         int comm_id = cs_cid[i];
         auto layer = mnet->layers()->get(std::string(cs_layer[i]));
         if (!layer) stop("cannot find layer " + std::string(cs_layer[i]) + " (community structure not compatible with this network?)");
-        auto actor = mnet->vertices()->get(std::string(cs_actor[i]));
+        auto actor = mnet->actors()->get(std::string(cs_actor[i]));
         if (!actor) stop("cannot find actor " + std::string(cs_actor[i]) + " (community structure not compatible with this network?)");
         
         auto iv = std::make_pair(actor, layer);
